;;; srfi-28.scm --- Basic Format Strings

;; Copyright (C) 2014 Free Software Foundation, Inc.
;;
;; This library is free software; you can redistribute it and/or
;; modify it under the terms of the GNU Lesser General Public
;; License as published by the Free Software Foundation; either
;; version 3 of the License, or (at your option) any later version.
;;
;; This library is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; Lesser General Public License for more details.
;;
;; You should have received a copy of the GNU Lesser General Public
;; License along with this library; if not, write to the Free Software
;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

;;; Commentary:

;; This module provides a wrapper for simple-format that always outputs
;; to a string.
;;
;; This module is documented in the Guile Reference Manual.

;;; Code:

(define-module (srfi srfi-28)
  #:replace (format))

(define (format message . args)
  (apply simple-format #f message args))

(cond-expand-provide (current-module) '(srfi-28))
